"use strict";
/// <reference types="webpack-env" />
Object.defineProperty(exports, "__esModule", { value: true });
var IntlChunkLoader = /** @class */ (function () {
    function IntlChunkLoader(manifest, chunks, cached) {
        if (chunks === void 0) { chunks = []; }
        if (cached === void 0) { cached = {}; }
        var _this = this;
        this.load = function (chunk, locale) {
            var loadedChunks = _this.loadedChunksPerLocale.get(locale);
            var src = _this.manifest[chunk] && _this.manifest[chunk][locale];
            if (!src || (loadedChunks && loadedChunks.has(chunk))) {
                return Promise.resolve();
            }
            return new Promise(function (resolve, reject) {
                var request = new XMLHttpRequest();
                request.open('GET', src);
                request.addEventListener('load', function () {
                    var _a;
                    try {
                        _this.updateCache((_a = {},
                            _a[locale] = JSON.parse(request.responseText),
                            _a));
                        _this.setChunkAsLoaded(chunk, locale);
                    }
                    catch (e) {
                        reject(e);
                    }
                    resolve();
                });
                request.addEventListener('error', reject);
                request.send();
            });
        };
        this.manifest = manifest;
        this.chunksToLoad = new Set(chunks);
        this.cache = {};
        this.loadedChunksPerLocale = new Map();
        if (cached) {
            Object.keys(cached).forEach(function (chunk) {
                var cachedMessages = cached[chunk];
                _this.updateCache(cachedMessages);
                Object.keys(cachedMessages).forEach(function (locale) {
                    _this.setChunkAsLoaded(chunk, locale);
                });
            });
        }
    }
    IntlChunkLoader.prototype.updateCache = function (messages) {
        var _this = this;
        Object.keys(messages).forEach(function (locale) {
            if (!_this.cache[locale]) {
                _this.cache[locale] = {};
            }
            Object.assign(_this.cache[locale], messages[locale]);
        });
    };
    IntlChunkLoader.prototype.setChunkAsLoaded = function (chunk, locale) {
        var set = this.loadedChunksPerLocale.get(locale);
        if (set) {
            set.add(chunk);
        }
        else {
            this.loadedChunksPerLocale.set(locale, new Set([chunk]));
        }
    };
    IntlChunkLoader.prototype.loadPending = function (locale) {
        var _this = this;
        var size = this.chunksToLoad.size;
        return Promise.all(Array.from(this.chunksToLoad).map(function (chunk) { return _this.load(chunk, locale); }))
            .then(function () {
            if (_this.chunksToLoad.size > size) {
                return _this.loadPending(locale);
            }
        });
    };
    IntlChunkLoader.prototype.setLocale = function (locale) {
        var _this = this;
        return this.loadPending(locale)
            .then(function () {
            _this.locale = locale;
        });
    };
    IntlChunkLoader.prototype.push = function (chunk) {
        this.chunksToLoad.add(chunk);
        if (this.locale) {
            return this.load(chunk, this.locale);
        }
        return Promise.resolve();
    };
    IntlChunkLoader.prototype.hot = function (manifest, messages) {
        Object.assign(this.manifest, manifest);
        this.updateCache(messages);
    };
    Object.defineProperty(IntlChunkLoader.prototype, "messages", {
        get: function () {
            if (this.locale) {
                return this.cache[this.locale];
            }
            return undefined;
        },
        enumerable: true,
        configurable: true
    });
    return IntlChunkLoader;
}());
exports.IntlChunkLoader = IntlChunkLoader;
function initialize(cached) {
    if (!__webpack_require__.intlManifest) {
        throw new Error('Manifest generated by react-intl-extract-webpack-plugin not found. Make sure to use this plugin in your webpack config.');
    }
    if (!(__webpack_require__.intlChunkLoader instanceof IntlChunkLoader)) {
        __webpack_require__.intlChunkLoader = new IntlChunkLoader(__webpack_require__.intlManifest, __webpack_require__.intlChunkLoader, cached);
    }
    else {
        throw new Error('Initialize function for intl chunk loader should be called only once.');
    }
    return __webpack_require__.intlChunkLoader;
}
exports.default = initialize;
